package cn.x47.handle;


import cn.x47.model.RIPEntry;
import cn.x47.model.RIPPacket;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToMessageDecoder;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 解码器
 */
public class RIPPacketDecoder extends MessageToMessageDecoder<DatagramPacket> {

    @Override
    protected void decode(ChannelHandlerContext ctx, DatagramPacket packet, List<Object> out) throws Exception {
        ByteBuf buf = packet.content();
        RIPPacket ripPacket = new RIPPacket();
        ripPacket.setCommand(buf.readByte());
        ripPacket.setVersion(buf.readByte());
        ripPacket.setUnused(buf.readShort());

        List<RIPEntry> entries = new ArrayList<>();
        while (buf.readableBytes() >= 20) {
            RIPEntry entry = new RIPEntry();
            entry.setAddressFamily(buf.readShort());
            entry.setRouteTag(buf.readShort());

            byte[] ipBytes = new byte[4];
            buf.readBytes(ipBytes);
            entry.setIpAddress(InetAddress.getByAddress(ipBytes));

            buf.readBytes(ipBytes);
            entry.setSubnetMask(InetAddress.getByAddress(ipBytes));
            buf.readBytes(ipBytes);
            entry.setNextHop(InetAddress.getByAddress(ipBytes));
            if (ripPacket.getVersion() == 1) {
                // 对于 RIP v1，子网掩码和下一跳需要推断或设为默认值
                entry.setSubnetMask(InetAddress.getByName("255.255.255.0")); // 示例，实际应根据 IP 类推断

            }

            byte[] address = entry.getNextHop().getAddress();
            byte[] address1 = InetAddress.getByName("0.0.0.0").getAddress();

            if(Arrays.equals(address, address1)) {
                entry.setNextHop(packet.sender().getAddress());
            }

            entry.setMetric((int) buf.readUnsignedInt());
            entries.add(entry);
        }
        ripPacket.setEntries(entries);
        out.add(ripPacket);
    }
}
