import java.net.InetAddress;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class RipV1PacketBuilder {

    public static byte[] buildRipV1RequestPacket() throws IOException {
        // 使用 ByteArrayOutputStream 和 DataOutputStream 来构建数据包
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(baos);

        // 1. Command (8比特) - 请求报文，值为 1
        dos.writeByte(1);  // 请求报文

        // 2. Version (8比特) - RIP v1，值为 1
        dos.writeByte(1);  // RIP v1

        // 3. Must be zero (16比特) - 保留字段，必须为零
        dos.writeShort(0); // 保留字段，填充为零

        // 4. Address Family Identifier (AFI) - 请求报文中为 0
        dos.writeShort(0); // 请求报文的 AFI 为 0

        // 5. IP Address (32比特) - 目标 IP 地址，RIP 请求报文中通常为 0.0.0.0
        InetAddress byName = InetAddress.getByName("200.0.2.0");

        // 6. Metric (32比特) - 请求报文的 Metric 为 16，表示不可达
        dos.writeInt(16); // Metric = 16

        // 转换成字节数组
        return baos.toByteArray();
    }

    public static byte[] buildRipV1ResponsePacket() throws IOException {
        // 使用 ByteArrayOutputStream 和 DataOutputStream 来构建数据包
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(baos);

        // 1. Command (8比特) - 响应报文，值为 2
        dos.writeByte(2);  // 响应报文

        // 2. Version (8比特) - RIP v1，值为 1
        dos.writeByte(1);  // RIP v1

        // 3. Must be zero (16比特) - 保留字段，必须为零
        dos.writeShort(0); // 保留字段，填充为零

        // 4. Address Family Identifier (AFI) - 请求报文中为 0
        dos.writeShort(2); // 响应报文的 AFI 对于 IPv4 为 2

        dos.writeShort(0);

        // 5. IP Address (32比特) - 目标 IP 地址，假设响应报文中为某个 IP
        InetAddress byName = InetAddress.getByName("200.0.2.0");
        printByteArray(byName.getAddress());

        dos.write(byName.getAddress()); // 192.168.0.1（IPv4 地址）

        // 6. Metric (32比特) - 路由的 Metric
        dos.writeInt(1); // 假设 Metric 为 1

        // 转换成字节数组
        return baos.toByteArray();
    }

    public static void main(String[] args) throws IOException {
        byte[] requestPacket = buildRipV1RequestPacket();
        byte[] responsePacket = buildRipV1ResponsePacket();

        System.out.println("RIP v1 Request Packet: ");
        printByteArray(requestPacket);

        System.out.println("RIP v1 Response Packet: ");
        printByteArray(responsePacket);
    }

    // 打印字节数组的辅助函数
    private static void printByteArray(byte[] byteArray) {
        for (byte b : byteArray) {
            System.out.printf("%02X ", b);
        }
        System.out.println();
    }
}
